@@ -37,49 +37,45 @@
 import math
 
 import pyglet
-from pyglet.gl import *
 
-import reader
+# # Default to OpenAL if available:
+# pyglet.options['audio'] = 'openal', 'pulse', 'directsound', 'silent'
+
+from pyglet.gl import *
+from pyglet.graphics import Group
+from pyglet.math import Mat4, Vec3
+
+from . import reader
+
 
 pyglet.resource.path.append('res')
 pyglet.resource.reindex()
 
-# Default to OpenAL if available:
-pyglet.options['audio'] = 'openal', 'pulse', 'directsound', 'silent'
-
-
-def disc(r, x, y, slices=20, start=0, end=2*math.pi):
-    d = (end - start) / (slices - 1)
-    s = start
-    points = [(x, y)] + [(x + r * math.cos(a*d+s), y + r * math.sin(a*d+s))
-                         for a in range(slices)]
-    points = ((GLfloat * 2) * len(points))(*points)
-    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
-    glEnableClientState(GL_VERTEX_ARRAY)
-    glVertexPointer(2, GL_FLOAT, 0, points)
-    glDrawArrays(GL_TRIANGLE_FAN, 0, len(points))
-    glPopClientAttrib()
-
-
-def circle(r, x, y, slices=20):
-    d = 2 * math.pi / slices
-    points = [(x + r * math.cos(a*d), y + r * math.sin(a*d)) for a in range(slices)]
-    points = ((GLfloat * 2) * len(points))(*points)
-    glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
-    glEnableClientState(GL_VERTEX_ARRAY)
-    glVertexPointer(2, GL_FLOAT, 0, points)
-    glDrawArrays(GL_LINE_LOOP, 0, len(points))
-    glPopClientAttrib()
-
 
 def orientation_angle(orientation):
     return math.atan2(orientation[2], orientation[0])
 
 
+# Shape hack functions
+def _update_line(line, x, y, x2, y2):
+    line._x = x
+    line._y = y
+    line._x2 = x2
+    line._y2 = y2
+    line._update_translation()
+    line._update_vertices()
+
+def _bordered_rect_alpha(rect, inner_alpha, border_alpha):
+    rect._rgba = rect._rgba[:3] + (inner_alpha,)
+    rect._border_rgba = rect._border_rgba[:3] + (border_alpha,)
+    rect._update_color()
+
+
 class Handle:
     tip = ''
 
-    def __init__(self, player):
+    def __init__(self, window, player):
+        self.win = window
         self.player = player
 
     def hit_test(self, x, y, z):
@@ -87,11 +83,16 @@ class Handle:
         if dx * dx + dy * dy + dz * dz < self.radius * self.radius:
             return -dx, -dy, -dz
 
-    def draw(self):
+    def pos(self):
+        raise NotImplementedError()
+
+    def delete(self):
         pass
 
-    def begin_drag(self, window, offset):
-        self.win = window
+    def update_shapes(self):
+        pass
+
+    def begin_drag(self, offset):
         self.offset = offset
         return self
 
@@ -103,47 +104,43 @@ class Handle:
 
 
 class LabelHandle(Handle):
-    def __init__(self, player):
-        super(LabelHandle, self).__init__(player)
-        self.text = pyglet.text.Label('', font_size=10, color=(0, 0, 0, 255),
-                                      anchor_y='top', anchor_x='center')
+    def __init__(self, window, player):
+        super().__init__(window, player)
+        txt = getattr(player, 'label', '')
+        self.text = pyglet.text.Label(txt, font_size=10, color=(0, 0, 0, 255),
+                                      anchor_y='top', anchor_x='center', batch=window.text_batch)
+
+    def delete(self):
+        self.text.delete()
+
+    def update_shapes(self):
+        x, z = self.win.player_transform(self.player)
+        self.text.position = x, z - 5, 0
 
     def hit_test(self, x, y, z):
         return None
 
-    def draw(self):
-        if hasattr(self.player, 'label'):
-            x, _, y = self.player.position
-
-            # ech. fudge scale back to 1
-            mat = (GLfloat * 16)()
-            glGetFloatv(GL_MODELVIEW_MATRIX, mat)
-
-            glPushMatrix()
-            glTranslatef(x, y, 0)
-            glScalef(1/mat[0], 1/mat[5], 1/mat[10])
-            glTranslatef(0, -5, 0)
-
-            self.text.text = self.player.label
-            self.text.draw()
-
-            glPopMatrix()
-
 
 class PositionHandle(Handle):
     tip = 'position'
     radius = .3
 
-    def draw(self):
-        glPushMatrix()
-        glTranslatef(self.player.position[0], self.player.position[2], 0)
-        glColor3f(1, 0, 0)
-        glBegin(GL_TRIANGLES)
-        glVertex2f(0, self.radius)
-        glVertex2f(-self.radius * math.sqrt(3) / 2, -.5 * self.radius)
-        glVertex2f(self.radius * math.sqrt(3) / 2, -.5 * self.radius)
-        glEnd()
-        glPopMatrix()
+    def __init__(self, window, player):
+        super().__init__(window, player)
+        self._triangle = pyglet.shapes.Triangle(
+            0, self.radius,
+            -self.radius * math.sqrt(3.0) / 2.0, -self.radius / 2.0,
+            self.radius * math.sqrt(3.0) / 2.0, -self.radius / 2.0,
+            (255, 0, 0, 255),
+            window.handle_batch,
+            window.hgrp_position)
+        self._triangle.anchor_position = 0.0, -self.radius * 0.75
+
+    def update_shapes(self):
+        self._triangle.position = self.player.position[0], self.player.position[2]
+
+    def delete(self):
+        self._triangle.delete()
 
     def pos(self):
         return self.player.position
@@ -160,6 +157,17 @@ class OrientationHandle(Handle):
     radius = .1
     length = 1.5
 
+    def __init__(self, window, player):
+        super().__init__(window, player)
+        self._line = pyglet.shapes.Line(0, 0, 0, 0,
+                                        color=(74, 74, 74),
+                                        batch=window.handle_batch,
+                                        group=Group(0, window.hgrp_orientation))
+        self._disc = pyglet.shapes.Circle(0, 0, self.radius,
+                                          color=(255, 255, 0, 255),
+                                          batch=window.handle_batch,
+                                          group=Group(1, window.hgrp_orientation))
+
     def pos(self):
         x, _, z = self.player.position
         direction = self.get_orientation()
@@ -169,26 +177,21 @@ class OrientationHandle(Handle):
             z += direction[2] / sz * self.length
         return x, 0, z
 
-    def draw(self):
-        glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
+    def get_orientation(self):
+        return NotImplementedError()
 
+    def update_shapes(self):
         px, _, py = self.player.position
         x, _, y = self.pos()
 
-        # Dashed line
-        glColor3f(.3, .3, .3)
-        glEnable(GL_LINE_STIPPLE)
-        glLineStipple(1, 0x7777)
-        glBegin(GL_LINES)
-        glVertex2f(px, py)
-        glVertex2f(x, y)
-        glEnd()
+        self._line._width = 1.0 / self.win.zoom
+        _update_line(self._line, x, y, px, py)
 
-        # This handle (orientation)
-        glColor3f(1, 1, 0)
-        disc(self.radius, x, y)
+        self._disc.position = x, y
 
-        glPopAttrib()
+    def delete(self):
+        self._line.delete()
+        self._disc.delete()
 
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         px, py, pz = self.player.position
@@ -220,8 +223,14 @@ class ForwardOrientationHandle(OrientationHandle):
 
 
 class ConeAngleHandle(Handle):
+    length = .1
     radius = .1
 
+    def __init__(self, window, player):
+        super().__init__(window, player)
+        self._sector = pyglet.shapes.Sector(0, 0, self.length, color=self.fill_color, batch=window.handle_batch)
+        self._handle = pyglet.shapes.Circle(0, 0, self.radius, color=self.color, batch=window.handle_batch)
+
     def pos(self):
         px, py, pz = self.player.position
         angle = orientation_angle(self.player.cone_orientation)
@@ -230,25 +239,25 @@ class ConeAngleHandle(Handle):
         z = math.sin(angle) * self.length
         return px + x, py, pz + z
 
-    def draw(self):
-        glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
+    def get_angle(self):
+        raise NotImplementedError()
 
-        # Fill
-        glEnable(GL_BLEND)
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
-        glColor4f(*self.fill_color)
+    def update_shapes(self):
         px, _, py = self.player.position
         angle = orientation_angle(self.player.cone_orientation)
-        a = self.get_angle() * math.pi / 180.
-        disc(self.length, px, py,
-             start=angle - a/2,
-             end=angle + a/2)
+        a = math.radians(self.get_angle())
+
+        self._sector.angle = a
+        self._sector.rotation = math.degrees((-angle / 2.0) + (a / 4.0))
+        self._sector.position = px, py
 
-        # Handle
         x, _, y = self.pos()
-        glColor4f(*self.color)
-        disc(self.radius, x, y)
-        glPopAttrib()
+        self._handle.radius = self.radius
+        self._handle.position = x, y
+
+    def delete(self):
+        self._sector.delete()
+        self._handle.delete()
 
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         px, py, pz = self.player.position
@@ -264,8 +273,8 @@ class ConeAngleHandle(Handle):
 class ConeInnerAngleHandle(ConeAngleHandle):
     tip = 'cone_inner_angle'
     length = 1.
-    color = (.2, .8, .2, 1)
-    fill_color = (0, 1, 0, .1)
+    color = (51, 204, 51, 255)
+    fill_color = (0, 255, 0, 26)
 
     def get_angle(self):
         return self.player.cone_inner_angle
@@ -277,8 +286,8 @@ class ConeInnerAngleHandle(ConeAngleHandle):
 class ConeOuterAngleHandle(ConeAngleHandle):
     tip = 'cone_outer_angle'
     length = 1.2
-    color = (.2, .2, .8, 1)
-    fill_color = (0, 0, 1, .1)
+    color = (51, 51, 204, 255)
+    fill_color = (0, 0, 255, 26)
 
     def get_angle(self):
         return self.player.cone_outer_angle
@@ -295,56 +304,70 @@ class MoreHandle(Handle):
     open_width = 1.5
     open_height = 1.5
 
+    def __init__(self, window, player):
+        super().__init__(window, player)
+        self._box = pyglet.shapes.BorderedRectangle(0, 0, self.open_width, self.open_height, 1,
+                                                    (255, 255, 255), (0, 0, 0),
+                                                    window.handle_batch,
+                                                    Group(0, window.hgrp_label_more_collapsed))
+        # The BorderedRectangle vehemently defends itself from having differing alpha values,
+        # but hack them in cause we want them
+        _bordered_rect_alpha(self._box, 201, 255)
+
+        self._circ = pyglet.shapes.Circle(0, 0, self.radius,
+                                          batch=window.handle_batch,
+                                          group=Group(0, window.hgrp_label_more_collapsed))
+        self._outline = pyglet.shapes.Arc(0, 0, self.radius,
+                                          color=(0, 0, 0, 255),
+                                          batch=window.handle_batch,
+                                          group=Group(1, window.hgrp_label_more_collapsed))
+        self._line0 = pyglet.shapes.Line(0, 0, 0, 0,
+                                         color=(0, 0, 0, 255),
+                                         batch=window.handle_batch,
+                                         group=Group(2, window.hgrp_label_more_collapsed))
+        self._line1 = pyglet.shapes.Line(0, 0, 0, 0,
+                                         color=(0, 0, 0, 255),
+                                         batch=window.handle_batch,
+                                         group=Group(3, window.hgrp_label_more_collapsed))
+
     def pos(self):
         x, y, z = self.player.position
         return x + 1, y, z + 1
 
-    def draw(self):
+    def get_angle(self):
+        raise NotImplementedError()
+
+    def update_shapes(self):
+        self._box.visible = self.open
+        self._circ.visible = not self.open
+        self._outline.visible = not self.open
+        self._line0.visible = not self.open
+        self._line1.visible = not self.open
+
         x, _, z = self.pos()
-
         if self.open:
-            x -= .2
-            z += .2
-            glPushAttrib(GL_ENABLE_BIT)
-            glEnable(GL_BLEND)
-
-            glColor4f(1, 1, 1, .8)
-            glBegin(GL_QUADS)
-            glVertex2f(x, z)
-            glVertex2f(x + self.open_width, z)
-            glVertex2f(x + self.open_width, z - self.open_height)
-            glVertex2f(x, z - self.open_height)
-            glEnd()
-
-            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
-            glColor4f(0, 0, 0, 1)
-            glBegin(GL_QUADS)
-            glVertex2f(x, z)
-            glVertex2f(x + self.open_width, z)
-            glVertex2f(x + self.open_width, z - self.open_height)
-            glVertex2f(x, z - self.open_height)
-            glEnd()
-            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL)
-
-            glPopAttrib()
+            x -= 0.2
+            z += 0.2
+            self._box.border = 1.0 / self.win.zoom
+            self._box.position = x, z - self.open_height
         else:
-            glColor3f(1, 1, 1)
-            disc(self.radius, x, z)
-
-            glColor3f(0, 0, 0)
-            circle(self.radius, x, z)
-
+            self._circ.position = x, z
+            self._outline.position = x, z
             r = self.radius - 0.1
-            glBegin(GL_LINES)
-            glVertex2f(x - r, z)
-            glVertex2f(x + r, z)
-            glVertex2f(x, z - r)
-            glVertex2f(x, z + r)
-            glEnd()
+            self._line0._width = 1.0 / self.win.zoom
+            _update_line(self._line0, x - r, z, x + r, z)
+            self._line1._width = 1.0 / self.win.zoom
+            _update_line(self._line1, x, z - r, x, z + r)
 
-    def begin_drag(self, window, offset):
+    def delete(self):
+        self._box.delete()
+        self._circ.delete()
+        self._outline.delete()
+        self._line0.delete()
+        self._line1.delete()
+
+    def begin_drag(self, offset):
         self.open = True
-        self.win = window
         self.win.set_more_player_handles(self.player)
         return self
 
@@ -366,8 +389,16 @@ class SliderHandle(Handle):
     width = .05
     radius = .1
 
-    def __init__(self, player, x, z):
-        super(SliderHandle, self).__init__(player)
+    def __init__(self, window, player, x, z):
+        super().__init__(window, player)
+        self._groove = pyglet.shapes.Rectangle(0, 0, self.length, self.width,
+                                               color=(127, 127, 127, 255),
+                                               batch=window.handle_batch,
+                                               group=Group(0, window.hgrp_more_expanded))
+        self._thumb = pyglet.shapes.Circle(0, 0, self.radius,
+                                           color=(51, 51, 51, 255),
+                                           batch=window.handle_batch,
+                                           group=Group(1, window.hgrp_more_expanded))
         self.x = x
         self.z = z
 
@@ -377,23 +408,17 @@ class SliderHandle(Handle):
         z += self.z
         return x, y, z
 
-    def draw(self):
-        x = self.x + self.player.position[0]
-        z = self.z + self.player.position[2]
+    def update_shapes(self):
+        gx = self.x + self.player.position[0]
+        gz = self.z + self.player.position[2]
+        self._groove.position = gx, gz
 
-        # Groove
-        glColor3f(.5, .5, .5)
-        glBegin(GL_QUADS)
-        glVertex2f(x, z - self.width/2)
-        glVertex2f(x + self.length, z - self.width/2)
-        glVertex2f(x + self.length, z + self.width/2)
-        glVertex2f(x, z + self.width/2)
-        glEnd()
-
-        # Thumb
         x, _, z = self.pos()
-        glColor3f(.2, .2, .2)
-        disc(self.radius, x, z)
+        self._thumb.position = x, z
+
+    def delete(self):
+        self._groove.delete()
+        self._thumb.delete()
 
     def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
         px, py, pz = self.player.position
@@ -406,8 +431,8 @@ class SliderHandle(Handle):
 class VolumeHandle(SliderHandle):
     tip = 'volume'
 
-    def __init__(self, player):
-        super(VolumeHandle, self).__init__(player, 1, .9)
+    def __init__(self, window, player):
+        super().__init__(window, player, 1, .9)
 
     def get_value(self):
         return self.player.volume
@@ -419,8 +444,8 @@ class VolumeHandle(SliderHandle):
 class ListenerVolumeHandle(SliderHandle):
     tip = 'volume'
 
-    def __init__(self, player):
-        super(ListenerVolumeHandle, self).__init__(player, -.5, -1)
+    def __init__(self, window, player):
+        super().__init__(window, player, -.5, -1)
 
     def get_value(self):
         return self.player.volume
@@ -432,8 +457,8 @@ class ListenerVolumeHandle(SliderHandle):
 class MinDistanceHandle(SliderHandle):
     tip = 'min_distance'
 
-    def __init__(self, player):
-        super(MinDistanceHandle, self).__init__(player, 1, .6)
+    def __init__(self, window, player):
+        super().__init__(window, player, 1, .6)
 
     def get_value(self):
         return self.player.min_distance / 5.
@@ -445,8 +470,8 @@ class MinDistanceHandle(SliderHandle):
 class MaxDistanceHandle(SliderHandle):
     tip = 'max_distance'
 
-    def __init__(self, player):
-        super(MaxDistanceHandle, self).__init__(player, 1, .3)
+    def __init__(self, window, player):
+        super().__init__(window, player, 1, .3)
 
     def get_value(self):
         return min(self.player.max_distance / 5., 1.0)
@@ -458,8 +483,8 @@ class MaxDistanceHandle(SliderHandle):
 class ConeOuterGainHandle(SliderHandle):
     tip = 'cone_outer_gain'
 
-    def __init__(self, player):
-        super(ConeOuterGainHandle, self).__init__(player, 1, 0)
+    def __init__(self, window, player):
+        super().__init__(window, player, 1, 0)
 
     def get_value(self):
         return self.player.cone_outer_gain
@@ -470,69 +495,75 @@ class ConeOuterGainHandle(SliderHandle):
 
 class SoundSpaceWindow(pyglet.window.Window):
     def __init__(self, **kwargs):
-        kwargs.update(dict(
-            caption='Sound Space',
-            resizable=True,
-        ))
-        super(SoundSpaceWindow, self).__init__(**kwargs)
-
-        self.players = []
-        self.handles = []
-        self.more_handles = []
-
-        listener = pyglet.media.get_audio_driver().get_listener()
-        self.handles.append(PositionHandle(listener))
-        self.handles.append(ForwardOrientationHandle(listener))
-        self.handles.append(ListenerVolumeHandle(listener))
-        self.handles.append(LabelHandle(listener))
-
-        self.tip = pyglet.text.Label('', font_size=10, color=(0, 0, 0, 255),
-                                     anchor_y='top', anchor_x='center')
-        self.tip_player = None
+        kwargs.setdefault("caption", "Sound Space")
+        kwargs.setdefault("resizable", True)
+        super().__init__(**kwargs)
 
         # pixels per unit
         self.zoom = 40
         self.tx = self.width/2
         self.ty = self.height/2
 
+        # The handle batch is drawn with a greatly zoomed view matrix
+        self._grid_batch = pyglet.graphics.Batch()
+        self.handle_batch = pyglet.graphics.Batch()
+        self.text_batch = pyglet.graphics.Batch()
+
+        # Various handle groups for proper rendering order
+        self.hgrp_position = Group(0)
+        self.hgrp_orientation = Group(1)
+        self.hgrp_cone_inner = Group(2)
+        self.hgrp_cone_outer = Group(3)
+        self.hgrp_label_more_collapsed = Group(4)
+        self.hgrp_more_expanded = Group(5)
+
+        self._grid = []
+        self._refresh_grid()
+        self.players = []
+        self.handles = []
+        self.more_handles = []
+
+        listener = pyglet.media.get_audio_driver().get_listener()
+        self.handles.append(PositionHandle(self, listener))
+        self.handles.append(ForwardOrientationHandle(self, listener))
+        self.handles.append(ListenerVolumeHandle(self, listener))
+        self.handles.append(LabelHandle(self, listener))
+
+        self.tip = pyglet.text.Label('', font_size=10, color=(0, 0, 0, 255),
+                                     anchor_y='top', anchor_x='center')
+        self.tip_player = None
+
     def add_player(self, player):
         self.players.append(player)
-        self.handles.append(PositionHandle(player))
-        self.handles.append(ConeOrientationHandle(player))
-        self.handles.append(ConeInnerAngleHandle(player))
-        self.handles.append(ConeOuterAngleHandle(player))
-        self.handles.append(LabelHandle(player))
-        self.handles.append(MoreHandle(player))
+        self.handles.append(PositionHandle(self, player))
+        self.handles.append(ConeOrientationHandle(self, player))
+        self.handles.append(ConeInnerAngleHandle(self, player))
+        self.handles.append(ConeOuterAngleHandle(self, player))
+        self.handles.append(LabelHandle(self, player))
+        self.handles.append(MoreHandle(self, player))
 
     def set_more_player_handles(self, player):
         if player:
             self.more_handles = [
-                VolumeHandle(player),
-                MinDistanceHandle(player),
-                MaxDistanceHandle(player),
-                ConeOuterGainHandle(player),
+                VolumeHandle(self, player),
+                MinDistanceHandle(self, player),
+                MaxDistanceHandle(self, player),
+                ConeOuterGainHandle(self, player),
             ]
         else:
+            for h in self.more_handles:
+                h.delete()
             self.more_handles = []
 
-    def draw_background(self):
-        glLoadIdentity()
-        glPushAttrib(GL_CURRENT_BIT)
-        glColor3f(1, 1, 1)
-        glBegin(GL_LINES)
-        for i in range(0, self.width, self.zoom):
-            glVertex2f(i, 0)
-            glVertex2f(i, self.height)
-        for i in range(0, self.height, self.zoom):
-            glVertex2f(0, i)
-            glVertex2f(self.width, i)
-        glEnd()
-        glPopAttrib()
+    def _refresh_grid(self):
+        for line in self._grid:
+            line.delete()
+        self._grid.clear()
 
-    def camera_transform(self):
-        glLoadIdentity()
-        glTranslatef(self.tx, self.ty, 0)
-        glScalef(self.zoom, self.zoom, 1)
+        for x in range(0, self.width, self.zoom):
+            self._grid.append(pyglet.shapes.Line(x, 0, x, self.height, batch=self._grid_batch))
+        for y in range(0, self.height, self.zoom):
+            self._grid.append(pyglet.shapes.Line(0, y, self.width, y, batch=self._grid_batch))
 
     def mouse_transform(self, x, y):
         return (float(x - self.tx) / self.zoom,
@@ -554,13 +585,18 @@ class SoundSpaceWindow(pyglet.window.Window):
     def on_draw(self):
         glClearColor(.8, .8, .8, 1)
         self.clear()
-        self.draw_background()
 
-        glPushMatrix()
-        self.camera_transform()
         for handle in self.handles + self.more_handles:
-            handle.draw()
-        glPopMatrix()
+            handle.update_shapes()
+
+        self.view = Mat4()
+        self._grid_batch.draw()
+
+        self.view = Mat4().translate(Vec3(self.tx, self.ty, 0)).scale(Vec3(self.zoom, self.zoom, 0))
+        self.handle_batch.draw()
+
+        self.view = Mat4()
+        self.text_batch.draw()
 
         if self.tip_player:
             player_pos = self.player_transform(self.tip_player)
@@ -571,13 +607,29 @@ class SoundSpaceWindow(pyglet.window.Window):
     def on_mouse_scroll(self, x, y, dx, dy):
         self.zoom += dy * 10
         self.zoom = min(max(self.zoom, 10), 100)
+        self._refresh_grid()
 
     def on_mouse_press(self, x, y, button, modifiers):
         handle, offset = self.hit_test(x, y)
         if handle:
-            self.push_handlers(handle.begin_drag(self, offset))
+            self.push_handlers(handle.begin_drag(offset))
         else:
-            self.push_handlers(PanView(self))
+            self.push_handlers(
+                on_mouse_press=self._pan_on_mouse_press_or_release,
+                on_mouse_release=self._pan_on_mouse_press_or_release,
+                on_mouse_drag=self._pan_on_mouse_drag,
+            )
+
+    def _pan_on_mouse_press_or_release(self, _x, _y, _b, _m):
+        self.remove_handlers(
+            on_mouse_press=self._pan_on_mouse_press_or_release,
+            on_mouse_release=self._pan_on_mouse_press_or_release,
+            on_mouse_drag=self._pan_on_mouse_drag,
+        )
+
+    def _pan_on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
+        self.tx += dx
+        self.ty += dy
 
     def on_mouse_motion(self, x, y, dx, dy):
         handle, offset = self.hit_test(x, y)
@@ -589,21 +641,6 @@ class SoundSpaceWindow(pyglet.window.Window):
             self.tip.text = ''
 
 
-class PanView:
-    def __init__(self, window):
-        self.win = window
-
-    def on_mouse_release(self, x, y, button, modifiers):
-        self.win.remove_handlers(self)
-
-    def on_mouse_press(self, x, y, button, modifiers):
-        self.win.remove_handlers(self)
-
-    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
-        self.win.tx += dx
-        self.win.ty += dy
-
-
 if __name__ == '__main__':
     # We swap Y and Z, moving to left-handed system
     listener = pyglet.media.get_audio_driver().get_listener()
@@ -616,7 +653,7 @@ if __name__ == '__main__':
 
     w = SoundSpaceWindow()
     r = reader.SpaceReader(w)
-    r.read(pyglet.resource.file('space.txt'))
+    r.read(pyglet.resource.file('space.txt', 'r'))
     player_group = pyglet.media.PlayerGroup(w.players)
     player_group.play()
 
