@@ -1,60 +1,12 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
-
 """Precise framerate calculation function scheduling.
 
-Measuring time
-==============
+The :py:mod:`~pyglet.clock` module allows you to schedule functions
+to run periodically, or for one-shot future execution. pyglet's default
+event loop (:py:func:`~pyglet.app.run`) keeps an internal instance of
+a :py:class:`~pyglet.clock.Clock`, which is ticked automatically.
 
-The `tick` and `get_fps` functions can be used in conjunction to fulfil most
-games' basic requirements::
-
-    from pyglet import clock
-    while True:
-        dt = clock.tick()
-        # ... update and render ...
-        print(f"FPS is {clock.get_fps()}")
-
-The ``dt`` value returned gives the number of seconds (as a float) since the
-last "tick".
-
-The `get_fps` function averages the framerate over a sliding window of
-approximately 1 second.  (You can calculate the instantaneous framerate by
-taking the reciprocal of ``dt``).
-
-Always remember to `tick` the clock!
+.. note:: Some internal modules will schedule items on the clock. If you
+          are using a custom event loop, always remember to `tick` the clock!
 
 Scheduling
 ==========
@@ -69,18 +21,18 @@ You can schedule a function to be called every time the clock is ticked::
 The `schedule_interval` method causes a function to be called every "n"
 seconds::
 
-    clock.schedule_interval(callback, .5)   # called twice a second
+    clock.schedule_interval(callback, 0.5)   # called twice a second
 
 The `schedule_once` method causes a function to be called once "n" seconds
 in the future::
 
     clock.schedule_once(callback, 5)        # called in 5 seconds
 
-All of the `schedule` methods will pass on any additional args or keyword args
+All the `schedule` methods will pass on any additional args or keyword args
 you specify to the callback function::
 
     def move(dt, velocity, sprite):
-       sprite.position += dt * velocity
+        sprite.position += dt * velocity
 
     clock.schedule(move, velocity=5.0, sprite=alien)
 
@@ -96,14 +48,14 @@ The clock functions are all relayed to an instance of
 :py:class:`~pyglet.clock.Clock` which is initialised with the module.  You can
 get this instance to use directly::
 
-    clk = clock.get_default()
+    clk = pyglet.clock.get_default()
 
 You can also replace the default clock with your own:
 
-    myclk = clock.Clock()
-    clock.set_default(myclk)
+    myclk = pyglet.clock.Clock()
+    pyglet.clock.set_default(myclk)
 
-Each clock maintains its own set of scheduled functions and FPS
+Each clock maintains its own set of scheduled functions and frequency
 measurement.  Each clock must be "ticked" separately.
 
 Multiple and derived clocks potentially allow you to separate "game-time" and
@@ -111,13 +63,14 @@ Multiple and derived clocks potentially allow you to separate "game-time" and
 of the system clock.
 """
 
-import time
+import time as _time
 
-from operator import attrgetter
-from heapq import heappush, heappop, heappushpop
-from collections import deque
-
-from pyglet import compat_platform
+from typing import Callable
+from heapq import heappop as _heappop
+from heapq import heappush as _heappush
+from heapq import heappushpop as _heappushpop
+from operator import attrgetter as _attrgetter
+from collections import deque as _deque
 
 
 class _ScheduledItem:
@@ -148,25 +101,6 @@ class _ScheduledIntervalItem:
 
 
 class Clock:
-    """Class for calculating and limiting framerate.
-
-    It is also used for calling scheduled functions.
-    """
-
-    #: The minimum amount of time in seconds this clock will attempt to sleep
-    #: for when framerate limiting.  Higher values will increase the
-    #: accuracy of the limiting but also increase CPU usage while
-    #: busy-waiting.  Lower values mean the process sleeps more often, but is
-    #: prone to over-sleep and run at a potentially lower or uneven framerate
-    #: than desired.
-
-    #: On Windows, MIN_SLEEP is larger because the default timer resolution
-    #: is set by default to 15 .6 ms.
-    MIN_SLEEP = 0.008 if compat_platform in ('win32', 'cygwin') else 0.005
-
-    #: The amount of time in seconds this clock subtracts from sleep values
-    #: to compensate for lazy operating systems.
-    SLEEP_UNDERSHOOT = MIN_SLEEP - 0.001
 
     # List of functions to call every tick.
     _schedule_items = None
@@ -177,23 +111,20 @@ class Clock:
     # If True, a sleep(0) is inserted on every tick.
     _force_sleep = False
 
-    def __init__(self, time_function=time.perf_counter):
+    def __init__(self, time_function=_time.perf_counter):
         """Initialise a Clock, with optional custom time function.
 
-        :Parameters:
-            `time_function` : function
-                Function to return the elapsed time of the application,
-                in seconds.  Defaults to time.time, but can be replaced
-                to allow for easy time dilation effects or game pausing.
-
+        You can provide a custom time function to return the elapsed
+        time of the application, in seconds. Defaults to time.perf_counter,
+        but can be replaced to allow for easy time dilation effects or game
+        pausing.
         """
-        super(Clock, self).__init__()
         self.time = time_function
         self.next_ts = self.time()
         self.last_ts = None
 
-        # Used by self.get_fps to show update frequency
-        self.times = deque()
+        # Used by self.get_frequency to show update frequency
+        self.times = _deque()
         self.cumulative_time = 0
         self.window_size = 60
 
@@ -202,24 +133,19 @@ class Clock:
         self._current_interval_item = None
 
     @staticmethod
-    def sleep(microseconds):
-        time.sleep(microseconds * 1e-6)
+    def sleep(microseconds: float):
+        _time.sleep(microseconds * 1e-6)
 
-    def update_time(self):
+    def update_time(self) -> float:
         """Get the elapsed time since the last call to `update_time`.
 
         This updates the clock's internal measure of time and returns
-        the difference since the last update (or since the clock was created).
-
-        .. versionadded:: 1.2
-
-        :rtype: float
-        :return: The number of seconds since the last `update_time`, or 0
-                 if this was the first time it was called.
+        the difference (in seconds) since the last time it was called.
+        The first call of this method always returns 0.
         """
         ts = self.time()
         if self.last_ts is None:
-            delta_t = 0
+            delta_t = 0.0
         else:
             delta_t = ts - self.last_ts
             self.times.appendleft(delta_t)
@@ -230,9 +156,11 @@ class Clock:
 
         return delta_t
 
-    def call_scheduled_functions(self, dt):
+    def call_scheduled_functions(self, dt: float) -> bool:
         """Call scheduled functions that elapsed on the last `update_time`.
 
+        Returns True if any functions were called, otherwise False.
+
         .. versionadded:: 1.2
 
         :Parameters:
@@ -240,9 +168,6 @@ class Clock:
                 The elapsed time since the last update to pass to each
                 scheduled function.  This is *not* used to calculate which
                 functions have elapsed.
-
-        :rtype: bool
-        :return: True if any functions were called, otherwise False.
         """
         now = self.last_ts
         result = False  # flag indicates if any function was called
@@ -275,11 +200,11 @@ class Clock:
             # case it needs to be rescheduled.  it is more efficient
             # to push and pop the heap at once rather than two operations
             if item is None:
-                item = heappop(interval_items)
+                item = _heappop(interval_items)
             else:
-                item = heappushpop(interval_items, item)
+                item = _heappushpop(interval_items, item)
 
-            # a scheduled function may try and unschedule itself
+            # a scheduled function may try to unschedule itself,
             # so we need to keep a reference to the current
             # item no longer on heap to be able to check
             self._current_interval_item = item
@@ -301,15 +226,15 @@ class Clock:
 
                 # test the schedule for the next execution
                 if item.next_ts <= now:
-                    # the scheduled time of this item has already passed
-                    # so it must be rescheduled
+                    # the scheduled time of this item has already
+                    # passed, so it must be rescheduled
                     if now - item.next_ts < 0.05:
                         # missed execution time by 'reasonable' amount, so
                         # reschedule at normal interval
                         item.next_ts = now + item.interval
                     else:
                         # missed by significant amount, now many events have
-                        # likely missed execution. do a soft reschedule to
+                        # likely missed execution. do a soft re-schedule to
                         # avoid lumping many events together.
                         # in this case, the next dt will not be accurate
                         item.next_ts = get_soft_next_ts(now, item.interval)
@@ -319,14 +244,17 @@ class Clock:
                 self._current_interval_item = item = None
 
         if item is not None:
-            heappush(interval_items, item)
+            _heappush(interval_items, item)
 
         return True
 
-    def tick(self, poll=False):
+    def tick(self, poll=False) -> float:
         """Signify that one frame has passed.
 
-        This will call any scheduled functions that have elapsed.
+        This will call any scheduled functions that have elapsed,
+        and returns the number of seconds since the last time this
+        method has been called. The first time this method is called,
+        0 is returned.
 
         :Parameters:
             `poll` : bool
@@ -334,12 +262,6 @@ class Clock:
                 but will not sleep or busy-wait for any reason.  Recommended
                 for advanced applications managing their own sleep timers
                 only.
-
-                Since pyglet 1.1.
-
-        :rtype: float
-        :return: The number of seconds since the last "tick", or 0 if this was
-            the first frame.
         """
         if not poll and self._force_sleep:
             self.sleep(0)
@@ -348,7 +270,7 @@ class Clock:
         self.call_scheduled_functions(delta_t)
         return delta_t
 
-    def get_sleep_time(self, sleep_idle):
+    def get_sleep_time(self, sleep_idle: bool):
         """Get the time until the next item is scheduled.
 
         Applications can choose to continue receiving updates at the
@@ -382,29 +304,23 @@ class Clock:
 
         return None
 
-    def get_fps(self):
+    def get_frequency(self) -> float:
         """Get the average clock update frequency of recent history.
 
         The result is the average of a sliding window of the last "n" updates,
         where "n" is some number designed to cover approximately 1 second.
         This is **not** the Window redraw rate.
-
-        :rtype: float
-        :return: The measured updates per second.
         """
         if not self.cumulative_time:
             return 0
         return len(self.times) / self.cumulative_time
 
-    def _get_nearest_ts(self):
+    def _get_nearest_ts(self) -> float:
         """Get the nearest timestamp.
 
         Schedule from now, unless now is sufficiently close to last_ts, in
         which case use last_ts.  This clusters together scheduled items that
-        probably want to be scheduled together.  The old (pre 1.1.1)
-        behaviour was to always use self.last_ts, and not look at ts.  The
-        new behaviour is needed because clock ticks can now be quite
-        irregular, and span several seconds.
+        probably want to be scheduled together.
         """
         last_ts = self.last_ts or self.next_ts
         ts = self.time()
@@ -413,6 +329,7 @@ class Clock:
         return last_ts
 
     def _get_soft_next_ts(self, last_ts, interval):
+
         def taken(ts, e):
             """Check if `ts` has already got an item scheduled nearby."""
             # TODO this function is slow and called very often.
@@ -425,12 +342,12 @@ class Clock:
 
             return False
 
-        # sorted list is required required to produce expected results
+        # sorted list is required to produce expected results
         # taken() will iterate through the heap, expecting it to be sorted
-        # and will not always catch smallest value, so sort here.
+        # and will not always catch the smallest value, so sort here.
         # do not remove the sort key...it is faster than relaying comparisons
         # NOTE: do not rewrite as popping from heap, as that is super slow!
-        self._schedule_interval_items.sort(key=attrgetter('next_ts'))
+        self._schedule_interval_items.sort(key=_attrgetter('next_ts'))
 
         # Binary division over interval:
         #
@@ -467,19 +384,25 @@ class Clock:
                 return next_ts
 
     def schedule(self, func, *args, **kwargs):
-        """Schedule a function to be called every frame.
+        """Schedule a function to be called every tick.
 
-        The function should have a prototype that includes ``dt`` as the
-        first argument, which gives the elapsed time, in seconds, since the
-        last clock tick.  Any additional arguments given to this function
-        are passed on to the callback::
+        The scheduled function should have a prototype that includes ``dt``
+        as the first argument, which gives the elapsed time in seconds since
+        the last clock tick. Any additional args or kwargs given to this
+        method are passed on to the callback::
 
             def callback(dt, *args, **kwargs):
                 pass
 
         :Parameters:
             `func` : callable
-                The function to call each frame.
+                The function to call each tick.
+
+        .. note:: Functions scheduled using this method will be called
+                  every tick by the default pyglet event loop, which can
+                  lead to high CPU usage. It is usually better to use
+                  :py:meth:`~pyglet.clock.schedule_interval` unless
+                  this is desired.
         """
         item = _ScheduledItem(func, args, kwargs)
         self._schedule_items.append(item)
@@ -498,7 +421,7 @@ class Clock:
         last_ts = self._get_nearest_ts()
         next_ts = last_ts + delay
         item = _ScheduledIntervalItem(func, 0, last_ts, next_ts, args, kwargs)
-        heappush(self._schedule_interval_items, item)
+        _heappush(self._schedule_interval_items, item)
 
     def schedule_interval(self, func, interval, *args, **kwargs):
         """Schedule a function to be called every `interval` seconds.
@@ -518,14 +441,36 @@ class Clock:
         last_ts = self._get_nearest_ts()
         next_ts = last_ts + interval
         item = _ScheduledIntervalItem(func, interval, last_ts, next_ts, args, kwargs)
-        heappush(self._schedule_interval_items, item)
+        _heappush(self._schedule_interval_items, item)
+
+    def schedule_interval_for_duration(self, func, interval, duration, *args, **kwargs):
+        """Schedule a function to be called every `interval` seconds
+        (see `schedule_interval`) and unschedule it after `duration` seconds.
+
+        The callback function prototype is the same as for `schedule`.
+
+        :Parameters:
+            `func` : callable
+                The function to call when the timer lapses.
+            `interval` : float
+                The number of seconds to wait between each call.
+            `duration` : float
+                The number of seconds for which the function is scheduled.
+
+        """
+        # NOTE: unschedule wrapper that takes `dt` argument
+        def _unschedule(dt: float, _func: Callable) -> None:
+            self.unschedule(_func)
+
+        self.schedule_interval(func, interval, *args, **kwargs)
+        self.schedule_once(_unschedule, duration, func)
 
     def schedule_interval_soft(self, func, interval, *args, **kwargs):
         """Schedule a function to be called every ``interval`` seconds.
 
         This method is similar to `schedule_interval`, except that the
         clock will move the interval out of phase with other scheduled
-        functions so as to distribute CPU more load evenly over time.
+        functions in order to distribute CPU load more evenly.
 
         This is useful for functions that need to be called regularly,
         but not relative to the initial start time.  :py:mod:`pyglet.media`
@@ -552,7 +497,7 @@ class Clock:
         next_ts = self._get_soft_next_ts(self._get_nearest_ts(), interval)
         last_ts = next_ts - interval
         item = _ScheduledIntervalItem(func, interval, last_ts, next_ts, args, kwargs)
-        heappush(self._schedule_interval_items, item)
+        _heappush(self._schedule_interval_items, item)
 
     def unschedule(self, func):
         """Remove a function from the schedule.
@@ -568,9 +513,7 @@ class Clock:
         # clever remove item without disturbing the heap:
         # 1. set function to an empty lambda -- original function is not called
         # 2. set interval to 0               -- item will be removed from heap eventually
-        valid_items = set(item
-                          for item in self._schedule_interval_items
-                          if item.func == func)
+        valid_items = set(item for item in self._schedule_interval_items if item.func == func)
 
         if self._current_interval_item:
             if self._current_interval_item.func == func:
@@ -587,14 +530,10 @@ class Clock:
 _default = Clock()
 
 
-def set_default(default):
+def set_default(default) -> None:
     """Set the default clock to use for all module-level functions.
 
-    By default an instance of :py:class:`~pyglet.clock.Clock` is used.
-
-    :Parameters:
-        `default` : `Clock`
-            The default clock to use.
+    By default, an instance of :py:class:`~pyglet.clock.Clock` is used.
     """
     global _default
     _default = default
@@ -605,146 +544,49 @@ def get_default():
 
     Return the :py:class:`~pyglet.clock.Clock` instance that is used by all
     module-level clock functions.
-
-    :rtype: `Clock`
-    :return: The default clock.
     """
     return _default
 
 
-def tick(poll=False):
-    """Signify that one frame has passed on the default clock.
-
-    This will call any scheduled functions that have elapsed.
-
-    :Parameters:
-        `poll` : bool
-            If True, the function will call any scheduled functions
-            but will not sleep or busy-wait for any reason.  Recommended
-            for advanced applications managing their own sleep timers
-            only.
-
-            Since pyglet 1.1.
-
-    :rtype: float
-    :return: The number of seconds since the last "tick", or 0 if this was the
-        first frame.
-    """
+def tick(poll: bool = False) -> float:
+    """:see: :py:meth:`~pyglet.clock.Clock.tick`"""
     return _default.tick(poll)
 
 
-def get_sleep_time(sleep_idle):
-    """Get the time until the next item is scheduled on the default clock.
-
-    See `Clock.get_sleep_time` for details.
-
-    :Parameters:
-        `sleep_idle` : bool
-            If True, the application intends to sleep through its idle
-            time; otherwise it will continue ticking at the maximum
-            frame rate allowed.
-
-    :rtype: float
-    :return: Time until the next scheduled event in seconds, or ``None``
-        if there is no event scheduled.
-
-    .. versionadded:: 1.1
-    """
+def get_sleep_time(sleep_idle: bool) -> float:
+    """:see: :py:meth:`~pyglet.clock.Clock.get_sleep_time`"""
     return _default.get_sleep_time(sleep_idle)
 
 
-def get_fps():
-    """Get the average clock update frequency.
-
-    The result is the sliding average of the last "n" updates,
-    where "n" is some number designed to cover approximately 1
-    second. This is the internal clock update rate, **not** the
-    Window redraw rate. Platform events, such as moving the
-    mouse rapidly, will cause the clock to refresh more often.
-
-    :rtype: float
-    :return: The measured updates per second.
-    """
-    return _default.get_fps()
+def get_frequency() -> float:
+    """:see: :py:meth:`~pyglet.clock.Clock.get_frequency`"""
+    return _default.get_frequency()
 
 
-def schedule(func, *args, **kwargs):
-    """Schedule 'func' to be called every frame on the default clock.
-
-    The arguments passed to func are ``dt``, followed by any ``*args`` and
-    ``**kwargs`` given here.
-
-    :Parameters:
-        `func` : callable
-            The function to call each frame.
-    """
+def schedule(func: Callable, *args, **kwargs) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.schedule`"""
     _default.schedule(func, *args, **kwargs)
 
 
-def schedule_interval(func, interval, *args, **kwargs):
-    """Schedule ``func`` on the default clock every interval seconds.
-
-    The arguments passed to ``func`` are ``dt`` (time since last function
-    call), followed by any ``*args`` and ``**kwargs`` given here.
-
-    :Parameters:
-        `func` : callable
-            The function to call when the timer lapses.
-        `interval` : float
-            The number of seconds to wait between each call.
-    """
+def schedule_interval(func: Callable, interval: float, *args, **kwargs) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.schedule_interval`"""
     _default.schedule_interval(func, interval, *args, **kwargs)
 
+def schedule_interval_for_duration(func: Callable, interval: float, duration: float, *args, **kwargs) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.schedule_interval_for_duration`"""
+    _default.schedule_interval_for_duration(func, interval, duration, *args, **kwargs)
 
-def schedule_interval_soft(func, interval, *args, **kwargs):
-    """Schedule ``func`` on the default clock every interval seconds.
 
-    The clock will move the interval out of phase with other scheduled
-    functions so as to distribute CPU more load evenly over time.
-
-    The arguments passed to ``func`` are ``dt`` (time since last function
-    call), followed by any ``*args`` and ``**kwargs`` given here.
-
-    :see: `Clock.schedule_interval_soft`
-
-    .. versionadded:: 1.1
-
-    :Parameters:
-        `func` : callable
-            The function to call when the timer lapses.
-        `interval` : float
-            The number of seconds to wait between each call.
-
-    """
+def schedule_interval_soft(func: Callable, interval: float, *args, **kwargs) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.schedule_interval_soft`"""
     _default.schedule_interval_soft(func, interval, *args, **kwargs)
 
 
-def schedule_once(func, delay, *args, **kwargs):
-    """Schedule ``func`` to be called once after ``delay`` seconds.
-
-    This function uses the default clock. ``delay`` can be a float. The
-    arguments passed to ``func`` are ``dt`` (time since last function call),
-    followed by any ``*args`` and ``**kwargs`` given here.
-
-    If no default clock is set, the func is queued and will be scheduled
-    on the default clock as soon as it is created.
-
-    :Parameters:
-        `func` : callable
-            The function to call when the timer lapses.
-        `delay` : float
-            The number of seconds to wait before the timer lapses.
-    """
+def schedule_once(func: Callable, delay: float, *args, **kwargs) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.schedule_once`"""
     _default.schedule_once(func, delay, *args, **kwargs)
 
 
-def unschedule(func):
-    """Remove ``func`` from the default clock's schedule.
-
-    No error is raised if the ``func`` was never scheduled.
-
-    :Parameters:
-        `func` : callable
-            The function to remove from the schedule.
-    """
+def unschedule(func: Callable) -> None:
+    """:see: :py:meth:`~pyglet.clock.Clock.unschedule`"""
     _default.unschedule(func)
