@@ -1,49 +1,17 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
-
 # TODO Windows Vista: need to call SetProcessDPIAware?  May affect GDI+ calls as well as font.
 
 import math
 import warnings
 
 from sys import byteorder
+from typing import Optional
+
 import pyglet
 from pyglet.font import base
 from pyglet.font import win32query
 import pyglet.image
 from pyglet.libs.win32.constants import *
+from pyglet.libs.win32.context_managers import device_context
 from pyglet.libs.win32.types import *
 from pyglet.libs.win32 import _gdi32 as gdi32, _user32 as user32
 from pyglet.libs.win32 import _kernel32 as kernel32
@@ -59,31 +27,38 @@ def str_ucs2(text):
         text = text.encode('utf_16_le')   # explicit endian avoids BOM
     return create_string_buffer(text + '\0')
 
+
 _debug_dir = 'debug_font'
+
+
 def _debug_filename(base, extension):
     import os
     if not os.path.exists(_debug_dir):
         os.makedirs(_debug_dir)
-    name = '%s-%%d.%%s' % os.path.join(_debug_dir, base)
+    name = f'{os.path.join(_debug_dir, base)}-{{0}}.{{1}}'
     num = 1
-    while os.path.exists(name % (num, extension)):
+    while os.path.exists(name.format(num, extension)):
         num += 1
-    return name % (num, extension)
+    return name.format(num, extension)
+
 
 def _debug_image(image, name):
     filename = _debug_filename(name, 'png')
     image.save(filename)
-    _debug('Saved image %r to %s' % (image, filename))
+    _debug(f'Saved image {image} to {filename}')
+
 
 _debug_logfile = None
+
+
 def _debug(msg):
     global _debug_logfile
     if not _debug_logfile:
         _debug_logfile = open(_debug_filename('log', 'txt'), 'wt')
     _debug_logfile.write(msg + '\n')
 
-class Win32GlyphRenderer(base.GlyphRenderer):
 
+class Win32GlyphRenderer(base.GlyphRenderer):
 
     def __init__(self, font):
         self._bitmap = None
@@ -101,13 +76,15 @@ class Win32GlyphRenderer(base.GlyphRenderer):
 
         gdi32.SelectObject(self._dc, self.font.hfont)
 
-    def _create_bitmap(self, width, height):
+    def _create_bitmap(self, width, height) -> None:
         pass
 
-    def render(self, text):
+    def render(self, text: str) -> pyglet.font.base.Glyph:
         raise NotImplementedError('abstract')
 
+
 class GDIGlyphRenderer(Win32GlyphRenderer):
+
     def __del__(self):
         try:
             if self._dc:
@@ -117,18 +94,22 @@ class GDIGlyphRenderer(Win32GlyphRenderer):
         except:
             pass
 
-    def render(self, text):
+    def render(self, text: str) -> pyglet.font.base.Glyph:
         # Attempt to get ABC widths (only for TrueType)
         abc = ABC()
-        if gdi32.GetCharABCWidthsW(self._dc, 
-            ord(text), ord(text), byref(abc)):
-            width = abc.abcB 
+        if gdi32.GetCharABCWidthsW(
+            self._dc,
+            ord(text), ord(text), byref(abc)
+        ):
+            width = abc.abcB
             lsb = abc.abcA
             advance = abc.abcA + abc.abcB + abc.abcC
+
         else:
             width_buf = c_int()
-            gdi32.GetCharWidth32W(self._dc, 
-                ord(text), ord(text), byref(width_buf))
+            gdi32.GetCharWidth32W(
+                self._dc, ord(text), ord(text), byref(width_buf))
+
             width = width_buf.value
             lsb = 0
             advance = width
@@ -136,30 +117,30 @@ class GDIGlyphRenderer(Win32GlyphRenderer):
         # Can't get glyph-specific dimensions, use whole line-height.
         height = self._bitmap_height
         image = self._get_image(text, width, height, lsb)
-        
+
         glyph = self.font.create_glyph(image)
         glyph.set_bearings(-self.font.descent, lsb, advance)
 
         if _debug_font:
-            _debug('%r.render(%s)' % (self, text))
-            _debug('abc.abcA = %r' % abc.abcA)
-            _debug('abc.abcB = %r' % abc.abcB)
-            _debug('abc.abcC = %r' % abc.abcC)
-            _debug('width = %r' % width)
-            _debug('height = %r' % height)
-            _debug('lsb = %r' % lsb)
-            _debug('advance = %r' % advance)
-            _debug_image(image, 'glyph_%s' % text)
-            _debug_image(self.font.textures[0], 'tex_%s' % text)
+            _debug(f'{self}.render({text})')
+            _debug(f'abc.abcA = {abc.abcA}')
+            _debug(f'abc.abcB = {abc.abcB}')
+            _debug(f'abc.abcC = {abc.abcC}')
+            _debug(f'width = {width}')
+            _debug(f'height = {height}')
+            _debug(f'lsb = {lsb}')
+            _debug(f'advance = {advance}')
+            _debug_image(image, f'glyph_{text}')
+            _debug_image(self.font.textures[0], f'tex_{text}')
 
         return glyph
 
-    def _get_image(self, text, width, height, lsb):
+    def _get_image(self, text: str, width: int, height: int, lsb: int) -> pyglet.image.ImageData:
         # There's no such thing as a greyscale bitmap format in GDI.  We can
         # create an 8-bit palette bitmap with 256 shades of grey, but
         # unfortunately antialiasing will not work on such a bitmap.  So, we
         # use a 32-bit bitmap and use the red channel as OpenGL's alpha.
-    
+
         gdi32.SelectObject(self._dc, self._bitmap)
         gdi32.SelectObject(self._dc, self.font.hfont)
         gdi32.SetBkColor(self._dc, 0x0)
@@ -168,16 +149,20 @@ class GDIGlyphRenderer(Win32GlyphRenderer):
 
         # Draw to DC
         user32.FillRect(self._dc, byref(self._bitmap_rect), self._black)
-        gdi32.ExtTextOutA(self._dc, -lsb, 0, 0, None, text,
+        gdi32.ExtTextOutA(
+            self._dc, -lsb, 0, 0, None, text,
             len(text), None)
+
         gdi32.GdiFlush()
 
         # Create glyph object and copy bitmap data to texture
-        image = pyglet.image.ImageData(width, height, 
+        image = pyglet.image.ImageData(
+            width, height,
             'AXXX', self._bitmap_data, self._bitmap_rect.right * 4)
+
         return image
-        
-    def _create_bitmap(self, width, height):
+
+    def _create_bitmap(self, width: int, height: int) -> None:
         self._black = gdi32.GetStockObject(BLACK_BRUSH)
         self._white = gdi32.GetStockObject(WHITE_BRUSH)
 
@@ -193,11 +178,12 @@ class GDIGlyphRenderer(Win32GlyphRenderer):
         info.bmiHeader.biWidth = width
         info.bmiHeader.biHeight = height
         info.bmiHeader.biPlanes = 1
-        info.bmiHeader.biBitCount = 32 
+        info.bmiHeader.biBitCount = 32
         info.bmiHeader.biCompression = BI_RGB
 
         self._dc = gdi32.CreateCompatibleDC(None)
-        self._bitmap = gdi32.CreateDIBSection(None,
+        self._bitmap = gdi32.CreateDIBSection(
+            None,
             byref(info), DIB_RGB_COLORS, byref(data), None,
             0)
         # Spookiness: the above line causes a "not enough storage" error,
@@ -214,67 +200,97 @@ class GDIGlyphRenderer(Win32GlyphRenderer):
         self._bitmap_height = height
 
         if _debug_font:
-            _debug('%r._create_dc(%d, %d)' % (self, width, height))
-            _debug('_dc = %r' % self._dc)
-            _debug('_bitmap = %r' % self._bitmap)
-            _debug('pitch = %r' % pitch)
-            _debug('info.bmiHeader.biSize = %r' % info.bmiHeader.biSize)
+            _debug(f'{self}._create_dc({width}, {height})')
+            _debug(f'_dc = {self._dc}')
+            _debug(f'_bitmap = {self._bitmap}')
+            _debug(f'pitch = {pitch}')
+            _debug(f'info.bmiHeader.biSize = {info.bmiHeader.biSize}')
+
 
 class Win32Font(base.Font):
+
     glyph_renderer_class = GDIGlyphRenderer
 
-    def __init__(self, name, size, bold=False, italic=False, stretch=False, dpi=None):
+    def __init__(
+            self,
+            name: str, size: int,
+            bold: bool = False, italic: bool = False, stretch: bool = False,
+            dpi: Optional[float] = None
+    ):
         super(Win32Font, self).__init__()
 
         self.logfont = self.get_logfont(name, size, bold, italic, dpi)
         self.hfont = gdi32.CreateFontIndirectW(byref(self.logfont))
 
         # Create a dummy DC for coordinate mapping
-        dc = user32.GetDC(0)
-        metrics = TEXTMETRIC()
-        gdi32.SelectObject(dc, self.hfont)
-        gdi32.GetTextMetricsA(dc, byref(metrics))
-        self.ascent = metrics.tmAscent
-        self.descent = -metrics.tmDescent
-        self.max_glyph_width = metrics.tmMaxCharWidth
-        user32.ReleaseDC(0, dc)
+        with device_context(None) as dc:
+            metrics = TEXTMETRIC()
+            gdi32.SelectObject(dc, self.hfont)
+            gdi32.GetTextMetricsA(dc, byref(metrics))
+            self.ascent = metrics.tmAscent
+            self.descent = -metrics.tmDescent
+            self.max_glyph_width = metrics.tmMaxCharWidth
 
     def __del__(self):
         gdi32.DeleteObject(self.hfont)
 
     @staticmethod
-    def get_logfont(name, size, bold, italic, dpi):
+    def get_logfont(name: str, size: float, bold: bool, italic: bool, dpi: Optional[float] = None) -> LOGFONTW:
+        """Get a raw Win32 :py:class:`.LOGFONTW` struct for the given arguments.
+
+        Args:
+            name: The name of the font
+            size: The font size in points
+            bold: Whether to request the font as bold
+            italic: Whether to request the font as italic
+            dpi: The screen dpi
+
+        Returns:
+            LOGFONTW: a ctypes binding of a Win32 LOGFONTW struct
+        """
+
         # Create a dummy DC for coordinate mapping
-        dc = user32.GetDC(0)
-        if dpi is None:
-            dpi = 96
-        logpixelsy = dpi
+        with device_context(None) as dc:
+
+            # Default to 96 DPI unless otherwise specified
+            if dpi is None:
+                dpi = 96
+            log_pixels_y = dpi
+
+            # Create LOGFONTW font description struct
+            logfont = LOGFONTW()
+
+            # Convert point size to actual device pixels
+            logfont.lfHeight = int(-size * log_pixels_y // 72)
+
+            # Configure the LOGFONTW's font properties
+            if bold:
+                logfont.lfWeight = FW_BOLD
+            else:
+                logfont.lfWeight = FW_NORMAL
+            logfont.lfItalic = italic
+            logfont.lfFaceName = name
+            logfont.lfQuality = ANTIALIASED_QUALITY
 
-        logfont = LOGFONTW()
-        # Conversion of point size to device pixels
-        logfont.lfHeight = int(-size * logpixelsy // 72)
-        if bold:
-            logfont.lfWeight = FW_BOLD
-        else:
-            logfont.lfWeight = FW_NORMAL
-        logfont.lfItalic = italic
-        logfont.lfFaceName = name
-        logfont.lfQuality = ANTIALIASED_QUALITY
-        user32.ReleaseDC(0, dc)
         return logfont
 
     @classmethod
-    def have_font(cls, name):
+    def have_font(cls, name) -> bool:
         # [ ] add support for loading raster fonts
         return win32query.have_font(name)
 
     @classmethod
     def add_font_data(cls, data):
         numfonts = c_uint32()
-        gdi32.AddFontMemResourceEx(data, len(data), 0, byref(numfonts))
+        _handle = gdi32.AddFontMemResourceEx(data, len(data), 0, byref(numfonts))
+
+        # None means a null handle was returned, ie something went wrong
+        if _handle is None:
+            raise ctypes.WinError()
+
+
 
 # --- GDI+ font rendering ---
-
 from pyglet.image.codecs.gdiplus import PixelFormat32bppARGB, gdiplus, Rect
 from pyglet.image.codecs.gdiplus import ImageLockModeRead, BitmapData
 
@@ -293,6 +309,7 @@ StringFormatFlagsNoWrap = 0x00001000
 StringFormatFlagsLineLimit = 0x00002000
 StringFormatFlagsNoClip = 0x00004000
 
+
 class Rectf(ctypes.Structure):
     _fields_ = [
         ('x', ctypes.c_float),
@@ -301,6 +318,7 @@ class Rectf(ctypes.Structure):
         ('height', ctypes.c_float),
     ]
 
+
 class GDIPlusGlyphRenderer(Win32GlyphRenderer):
     def __del__(self):
         try:
@@ -318,9 +336,9 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
             pass
 
     def _create_bitmap(self, width, height):
-        self._data = (ctypes.c_byte * (4 * width * height))()
+        self._data = (BYTE * (4 * width * height))()
         self._bitmap = ctypes.c_void_p()
-        self._format = PixelFormat32bppARGB 
+        self._format = PixelFormat32bppARGB
         gdiplus.GdipCreateBitmapFromScan0(width, height, width * 4,
             self._format, self._data, ctypes.byref(self._bitmap))
 
@@ -335,11 +353,9 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
         gdiplus.GdipSetTextRenderingHint(self._graphics,
             TextRenderingHintAntiAliasGridFit)
 
-
         self._brush = ctypes.c_void_p()
         gdiplus.GdipCreateSolidFill(0xffffffff, ctypes.byref(self._brush))
 
-        
         self._matrix = ctypes.c_void_p()
         gdiplus.GdipCreateMatrix(ctypes.byref(self._matrix))
 
@@ -452,30 +468,33 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
 
         # Draw character to bitmap
         gdiplus.GdipGraphicsClear(self._graphics, 0x00000000)
-        gdiplus.GdipDrawString(self._graphics, 
+        gdiplus.GdipDrawString(self._graphics,
                                ch,
                                len_ch,
-                               self.font._gdipfont, 
-                               ctypes.byref(rect), 
+                               self.font._gdipfont,
+                               ctypes.byref(rect),
                                fmt,
                                self._brush)
+
         gdiplus.GdipFlush(self._graphics, 1)
         gdiplus.GdipDeleteStringFormat(fmt)
 
         bitmap_data = BitmapData()
-        gdiplus.GdipBitmapLockBits(self._bitmap, 
-            byref(self._rect), ImageLockModeRead, self._format, 
+        gdiplus.GdipBitmapLockBits(
+            self._bitmap,
+            byref(self._rect), ImageLockModeRead, self._format,
             byref(bitmap_data))
-        
+
         # Create buffer for RawImage
         buffer = create_string_buffer(
             bitmap_data.Stride * bitmap_data.Height)
         memmove(buffer, bitmap_data.Scan0, len(buffer))
-        
+
         # Unlock data
         gdiplus.GdipBitmapUnlockBits(self._bitmap, byref(bitmap_data))
-        
-        image = pyglet.image.ImageData(width, height,
+
+        image = pyglet.image.ImageData(
+            width, height,
             'BGRA', buffer, -bitmap_data.Stride)
 
         glyph = self.font.create_glyph(image)
@@ -484,11 +503,13 @@ class GDIPlusGlyphRenderer(Win32GlyphRenderer):
         glyph.set_bearings(-self.font.descent, lsb, advance)
         return glyph
 
+
 FontStyleBold = 1
 FontStyleItalic = 2
 UnitPixel = 2
 UnitPoint = 3
-        
+
+
 class GDIPlusFont(Win32Font):
     glyph_renderer_class = GDIPlusGlyphRenderer
 
@@ -511,6 +532,12 @@ class GDIPlusFont(Win32Font):
         self._name = name
 
         family = ctypes.c_void_p()
+
+        # GDI will add @ in front of a localized font for some Asian languages. However, GDI will also not find it
+        # based on that name (???). Here we remove it before checking font collections.
+        if name[0] == "@":
+            name = name[1:]
+
         name = ctypes.c_wchar_p(name)
 
         # Look in private collection first:
@@ -519,6 +546,9 @@ class GDIPlusFont(Win32Font):
 
         # Then in system collection:
         if not family:
+            if _debug_font:
+                print(f"Warning: Font '{name}' was not found. Defaulting to: {self._default_name}")
+
             gdiplus.GdipCreateFontFamilyFromName(name, None, ctypes.byref(family))
 
         # Nothing found, use default font.
@@ -579,6 +609,6 @@ class GDIPlusFont(Win32Font):
             gdiplus.GdipGetFamilyName(gpfamily, font_name, '\0')
             if font_name.value == name:
                 return True
-        
+
         # Else call parent class for system fonts
         return super(GDIPlusFont, cls).have_font(name)
