@@ -1,43 +1,6 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
-
 import io
 
-import pyglet
-
-from pyglet.media.exceptions import MediaException, CannotSeekException, MediaEncodeException
+from pyglet.media.exceptions import MediaException, CannotSeekException
 
 
 class AudioFormat:
@@ -125,7 +88,7 @@ class AudioData:
         length (int): Size of sample data, in bytes.
         timestamp (float): Time of the first sample, in seconds.
         duration (float): Total data duration, in seconds.
-        events (List[:class:`pyglet.media.events.MediaEvent`]): List of events
+        events (List[:class:`pyglet.media.drivers.base.MediaEvent`]): List of events
             contained within this packet. Events are timestamped relative to
             this audio packet.
     """
@@ -348,27 +311,11 @@ class Source:
                 attempted is raised.
 
         """
-        if not file:
-            file = open(filename, 'wb')
-
         if encoder:
-            encoder.encode(self, file, filename)
+            return encoder.enccode(self, filename, file)
         else:
-            first_exception = None
-            for encoder in pyglet.media.get_encoders(filename):
-
-                try:
-                    encoder.encode(self, file, filename)
-                    return
-                except MediaEncodeException as e:
-                    first_exception = first_exception or e
-                    file.seek(0)
-
-            if not first_exception:
-                raise MediaEncodeException(f"No Encoders are available for this extension: '{filename}'")
-            raise first_exception
-
-        file.close()
+            import pyglet.media.codecs
+            return pyglet.media.codecs.registry.encode(self, filename, file)
 
     # Internal methods that Player calls on the source:
 
