@@ -1,43 +1,10 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
 """High-level sound and video player."""
 
 import time
 from collections import deque
 
 import pyglet
+from pyglet.gl import GL_TEXTURE_2D
 from pyglet.media import buffered_logger as bl
 from pyglet.media.drivers import get_audio_driver
 from pyglet.media.codecs.base import Source, SourceGroup
@@ -142,7 +109,9 @@ class Player(pyglet.event.EventDispatcher):
         self._playlists = deque()
         self._audio_player = None
 
+        self._context = pyglet.gl.current_context
         self._texture = None
+
         # Desired play state (not an indication of actual state).
         self._playing = False
 
@@ -392,8 +361,7 @@ class Player(pyglet.event.EventDispatcher):
 
     def _create_texture(self):
         video_format = self.source.video_format
-        self._texture = pyglet.image.Texture.create(
-            video_format.width, video_format.height, rectangle=True)
+        self._texture = pyglet.image.Texture.create(video_format.width, video_format.height, GL_TEXTURE_2D)
         self._texture = self._texture.get_transform(flip_y=True)
         # After flipping the texture along the y axis, the anchor_y is set
         # to the top of the image. We want to keep it at the bottom.
@@ -427,6 +395,9 @@ class Player(pyglet.event.EventDispatcher):
         """
         return self.texture
 
+    def refill_buffer(self):
+        raise NotImplemented
+
     def seek_next_frame(self):
         """Step forwards one video frame in the current source."""
         time = self.source.get_next_video_timestamp()
@@ -486,9 +457,12 @@ class Player(pyglet.event.EventDispatcher):
 
         image = source.get_next_video_frame()
         if image is not None:
-            if self._texture is None:
-                self._create_texture()
-            self._texture.blit_into(image, 0, 0, 0)
+
+            with self._context:
+                if self._texture is None:
+                    self._create_texture()
+                self._texture.blit_into(image, 0, 0, 0)
+
         elif bl.logger is not None:
             bl.logger.log("p.P.ut.1.8")
 
@@ -634,7 +608,7 @@ class Player(pyglet.event.EventDispatcher):
     def on_driver_reset(self):
         """The audio driver has been reset, by default this will kill the current audio player and create a new one,
         and requeue the buffers. Any buffers that may have been queued in a player will be resubmitted.  It will
-        continue from from the last buffers submitted, not played and may cause sync issues if using video.
+        continue from the last buffers submitted, not played and may cause sync issues if using video.
 
         :event:
         """
