@@ -1,38 +1,3 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
-
 """Load application resources from a known path.
 
 Loading resources by specifying relative paths to filenames is often
@@ -85,12 +50,13 @@ The default path is ``['.']``.  If you modify the path, you must call
 .. versionadded:: 1.1
 """
 
-import io
 import os
 import sys
 import zipfile
 import weakref
 
+from io import BytesIO
+
 import pyglet
 
 
@@ -98,8 +64,18 @@ class ResourceNotFoundException(Exception):
     """The named resource was not found on the search path."""
 
     def __init__(self, name):
-        message = ('Resource "%s" was not found on the path.  '
-                   'Ensure that the filename has the correct captialisation.') % name
+        message = ("Resource '{}' was not found on the path.  "
+                   "Ensure that the filename has the correct capitalisation.".format(name))
+        Exception.__init__(self, message)
+
+
+class UndetectableShaderType(Exception):
+    """The type of the Shader source could not be identified."""
+
+    def __init__(self, name):
+        message = ("The Shader type of '{}' could not be determined.  "
+                   "Ensure that your source file has a standard extension, "
+                   "or provide a valid 'shader_type' parameter.".format(name))
         Exception.__init__(self, message)
 
 
@@ -173,16 +149,57 @@ def get_settings_path(name):
         if 'APPDATA' in os.environ:
             return os.path.join(os.environ['APPDATA'], name)
         else:
-            return os.path.expanduser('~/%s' % name)
+            return os.path.expanduser(f'~/{name}')
     elif pyglet.compat_platform == 'darwin':
-        return os.path.expanduser('~/Library/Application Support/%s' % name)
+        return os.path.expanduser(f'~/Library/Application Support/{name}')
     elif pyglet.compat_platform.startswith('linux'):
         if 'XDG_CONFIG_HOME' in os.environ:
             return os.path.join(os.environ['XDG_CONFIG_HOME'], name)
         else:
-            return os.path.expanduser('~/.config/%s' % name)
+            return os.path.expanduser(f'~/.config/{name}')
     else:
-        return os.path.expanduser('~/.%s' % name)
+        return os.path.expanduser(f'~/.{name}')
+
+
+def get_data_path(name):
+    """Get a directory to save user data.
+
+    For a Posix or Linux based system many distributions have a separate
+    directory to store user data for a specific application and this 
+    function returns the path to that location.  Note that the returned 
+    path may not exist: applications should use ``os.makedirs`` to 
+    construct it if desired.
+
+    On Linux, a directory `name` in the user's data directory is returned 
+    (usually under ``~/.local/share``).
+
+    On Windows (including under Cygwin) the `name` directory in the user's
+    ``Application Settings`` directory is returned.
+
+    On Mac OS X the `name` directory under ``~/Library/Application Support``
+    is returned.
+
+    :Parameters:
+        `name` : str
+            The name of the application.
+
+    :rtype: str
+    """
+
+    if pyglet.compat_platform in ('cygwin', 'win32'):
+        if 'APPDATA' in os.environ:
+            return os.path.join(os.environ['APPDATA'], name)
+        else:
+            return os.path.expanduser(f'~/{name}')
+    elif pyglet.compat_platform == 'darwin':
+        return os.path.expanduser(f'~/Library/Application Support/{name}')
+    elif pyglet.compat_platform.startswith('linux'):
+        if 'XDG_DATA_HOME' in os.environ:
+            return os.path.join(os.environ['XDG_DATA_HOME'], name)
+        else:
+            return os.path.expanduser(f'~/.local/share/{name}')
+    else:
+        return os.path.expanduser(f'~/.{name}')
 
 
 class Location:
@@ -214,14 +231,14 @@ class FileLocation(Location):
     """Location on the filesystem.
     """
 
-    def __init__(self, path):
+    def __init__(self, filepath):
         """Create a location given a relative or absolute path.
 
         :Parameters:
-            `path` : str
+            `filepath` : str
                 Path on the filesystem.
         """
-        self.path = path
+        self.path = filepath
 
     def open(self, filename, mode='rb'):
         return open(os.path.join(self.path, filename), mode)
@@ -254,7 +271,7 @@ class ZIPLocation(Location):
 
         forward_slash_path = path.replace(os.sep, '/')  # zip can only handle forward slashes
         text = self.zip.read(forward_slash_path)
-        return io.BytesIO(text)
+        return BytesIO(text)
 
 
 class URLLocation(Location):
@@ -275,7 +292,8 @@ class URLLocation(Location):
         self.base = base_url
 
     def open(self, filename, mode='rb'):
-        import urllib.parse, urllib.request
+        import urllib.parse
+        import urllib.request
         url = urllib.parse.urljoin(self.base, filename)
         return urllib.request.urlopen(url)
 
@@ -426,7 +444,7 @@ class Loader:
 
                 volume_index += 1
 
-            zip_stream = io.BytesIO(bytes_)
+            zip_stream = BytesIO(bytes_)
             if zipfile.is_zipfile(zip_stream):
                 return zip_stream
             else:
@@ -504,12 +522,12 @@ class Loader:
             file.close()
 
         if not atlas:
-            return img.get_texture(True)
+            return img.get_texture()
 
         # find an atlas suitable for the image
         bin = self._get_texture_atlas_bin(img.width, img.height, border)
         if bin is None:
-            return img.get_texture(True)
+            return img.get_texture()
 
         return bin.add(img, border)
 
@@ -678,6 +696,7 @@ class Loader:
                 return media.load(path, streaming=streaming)
             else:
                 file = location.open(name)
+
                 return media.load(name, file=file, streaming=streaming)
         except KeyError:
             raise ResourceNotFoundException(name)
@@ -757,8 +776,42 @@ class Loader:
         :rtype: `UnformattedDocument`
         """
         self._require_index()
-        file = self.file(name)
-        return pyglet.text.load(name, file, 'text/plain')
+        fileobj = self.file(name)
+        return pyglet.text.load(name, fileobj, 'text/plain')
+
+    def shader(self, name, shader_type=None):
+        """Load a Shader object.
+
+        :Parameters:
+            `name` : str
+                Filename of the Shader source to load.
+            `shader_type` : str
+                A hint for the type of shader, such as 'vertex', 'fragment', etc.
+                Not required if your shader has a standard file extension.
+
+        :rtype: A compiled `Shader` object.
+        """
+        # https://www.khronos.org/opengles/sdk/tools/Reference-Compiler/
+        shader_extensions = {'comp': "compute",
+                             'frag': "fragment",
+                             'geom': "geometry",
+                             'tesc': "tescontrol",
+                             'tese': "tesevaluation",
+                             'vert': "vertex"}
+        fileobj = self.file(name, 'r')
+        source_string = fileobj.read()
+
+        if not shader_type:
+            try:
+                _, extension = os.path.splitext(name)
+                shader_type = shader_extensions.get(extension.strip("."))
+            except KeyError:
+                raise UndetectableShaderType(name=name)
+
+        if shader_type not in shader_extensions.values():
+            raise UndetectableShaderType(name=name)
+
+        return pyglet.graphics.shader.Shader(source_string, shader_type)
 
     def get_cached_texture_names(self):
         """Get the names of textures currently cached.
@@ -805,6 +858,7 @@ texture = _default_loader.texture
 html = _default_loader.html
 attributed = _default_loader.attributed
 text = _default_loader.text
+shader = _default_loader.shader
 get_cached_texture_names = _default_loader.get_cached_texture_names
 get_cached_image_names = _default_loader.get_cached_image_names
 get_cached_animation_names = _default_loader.get_cached_animation_names
