@@ -1,37 +1,3 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2022 pyglet contributors
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
 from ctypes import *
 from functools import lru_cache
 import unicodedata
@@ -70,6 +36,8 @@ _motion_map = {
     (key.END, True): key.MOTION_END_OF_FILE,
     (key.BACKSPACE, False): key.MOTION_BACKSPACE,
     (key.DELETE, False): key.MOTION_DELETE,
+    (key.C, True): key.MOTION_COPY,
+    (key.V, True): key.MOTION_PASTE
 }
 
 
@@ -183,7 +151,8 @@ class Win32Window(BaseWindow):
                 self._get_window_proc(self._event_handlers))
             self._window_class.style = CS_VREDRAW | CS_HREDRAW | CS_OWNDC
             self._window_class.hInstance = 0
-            self._window_class.hIcon = _user32.LoadIconW(module, MAKEINTRESOURCE(1))
+            self._window_class.hIcon = _user32.LoadImageW(module, MAKEINTRESOURCE(1), IMAGE_ICON,
+                                                          0, 0, LR_DEFAULTSIZE | LR_SHARED)
             self._window_class.hbrBackground = black
             self._window_class.lpszMenuName = None
             self._window_class.cbClsExtra = 0
@@ -231,10 +200,6 @@ class Win32Window(BaseWindow):
                 self._view_window_class.hInstance,
                 0)
 
-            if not self._view_hwnd:
-                last_error = _kernel32.GetLastError()
-                raise Exception("Failed to create handle", self, last_error, self._view_hwnd, self._hwnd)
-
             self._dc = _user32.GetDC(self._view_hwnd)
 
             # Only allow files being dropped if specified.
@@ -293,9 +258,9 @@ class Win32Window(BaseWindow):
             self.context.attach(self.canvas)
             self._wgl_context = self.context._context
 
-        self.set_caption(self._caption)
-
         self.switch_to()
+
+        self.set_caption(self._caption)
         self.set_vsync(self._vsync)
 
         if self._visible:
@@ -318,14 +283,16 @@ class Win32Window(BaseWindow):
             super(Win32Window, self).close()
             return
 
+        self.set_mouse_platform_visible(True)
+
         _user32.DestroyWindow(self._hwnd)
+        _user32.UnregisterClassW(self._view_window_class.lpszClassName, 0)
         _user32.UnregisterClassW(self._window_class.lpszClassName, 0)
 
         self._window_class = None
         self._view_window_class = None
         self._view_event_handlers.clear()
         self._event_handlers.clear()
-        self.set_mouse_platform_visible(True)
         self._hwnd = None
         self._dc = None
         self._wgl_context = None
@@ -400,13 +367,11 @@ class Win32Window(BaseWindow):
         return point.x, point.y
 
     def set_size(self, width, height):
-        if self._fullscreen:
-            raise WindowException('Cannot set size of fullscreen window.')
+        super().set_size(width, height)
         width, height = self._client_to_window_size(width, height)
         _user32.SetWindowPos(self._hwnd, 0, 0, 0, width, height,
-                             (SWP_NOZORDER |
-                              SWP_NOMOVE |
-                              SWP_NOOWNERZORDER))
+                             (SWP_NOZORDER | SWP_NOMOVE | SWP_NOOWNERZORDER))
+        self.dispatch_event('on_resize', self._width, self._height)
 
     def get_size(self):
         # rect = RECT()
@@ -463,7 +428,7 @@ class Win32Window(BaseWindow):
             else:
                 cursor = self._create_cursor_from_image(self._mouse_cursor)
 
-            _user32.SetClassLongW(self._view_hwnd, GCL_HCURSOR, cursor)
+            _user32.SetClassLongPtrW(self._view_hwnd, GCL_HCURSOR, cursor)
             _user32.SetCursor(cursor)
 
         if platform_visible == self._mouse_platform_visible:
@@ -545,7 +510,7 @@ class Win32Window(BaseWindow):
 
         if exclusive and self._has_focus:
             _user32.RegisterHotKey(self._hwnd, 0, WIN32_MOD_ALT, VK_TAB)
-        else:
+        elif self._exclusive_keyboard and not exclusive:
             _user32.UnregisterHotKey(self._hwnd, 0)
 
         self._exclusive_keyboard = exclusive
@@ -688,6 +653,43 @@ class Win32Window(BaseWindow):
 
         return icon
 
+    def set_clipboard_text(self, text: str):
+        valid = _user32.OpenClipboard(self._view_hwnd)
+        if not valid:
+            return
+
+        _user32.EmptyClipboard()
+
+        size = (len(text) + 1) * sizeof(WCHAR)  # UTF-16
+
+        cb_data = _kernel32.GlobalAlloc(GMEM_MOVEABLE, size)
+        locked_data = _kernel32.GlobalLock(cb_data)
+        memmove(locked_data, text, size)  # Trying to encode in utf-16 causes garbled text. Accepts str fine?
+        _kernel32.GlobalUnlock(cb_data)
+
+        _user32.SetClipboardData(CF_UNICODETEXT, cb_data)
+
+        _user32.CloseClipboard()
+
+    def get_clipboard_text(self) -> str:
+        text = ''
+
+        valid = _user32.OpenClipboard(self._view_hwnd)
+        if not valid:
+            print("Could not open clipboard")
+            return ''
+
+        cb_obj = _user32.GetClipboardData(CF_UNICODETEXT)
+        if cb_obj:
+            locked_data = _kernel32.GlobalLock(cb_obj)
+            if locked_data:
+                text = ctypes.wstring_at(locked_data)
+
+                _kernel32.GlobalUnlock(cb_obj)
+
+        _user32.CloseClipboard()
+        return text
+
     # Private util
 
     def _client_to_window_size(self, width, height):
@@ -965,6 +967,10 @@ class Win32Window(BaseWindow):
             buttons |= mouse.MIDDLE
         if wParam & MK_RBUTTON:
             buttons |= mouse.RIGHT
+        if wParam & MK_XBUTTON1:
+            buttons |= mouse.MOUSE4
+        if wParam & MK_XBUTTON2:
+            buttons |= mouse.MOUSE5
 
         if buttons:
             # Drag event
@@ -1036,6 +1042,26 @@ class Win32Window(BaseWindow):
         return self._event_mousebutton(
             'on_mouse_release', mouse.RIGHT, lParam)
 
+    @ViewEventHandler
+    @Win32EventHandler(WM_XBUTTONDOWN)
+    def _event_xbuttondown(self, msg, wParam, lParam):
+        if c_short(wParam >> 16).value == 1:
+            button = mouse.MOUSE4
+        if c_short(wParam >> 16).value == 2:
+            button = mouse.MOUSE5
+        return self._event_mousebutton(
+            'on_mouse_press', button, lParam)
+
+    @ViewEventHandler
+    @Win32EventHandler(WM_XBUTTONUP)
+    def _event_xbuttonup(self, msg, wParam, lParam):
+        if c_short(wParam >> 16).value == 1:
+            button = mouse.MOUSE4
+        if c_short(wParam >> 16).value == 2:
+            button = mouse.MOUSE5
+        return self._event_mousebutton(
+            'on_mouse_release', button, lParam)
+
     @Win32EventHandler(WM_MOUSEWHEEL)
     def _event_mousewheel(self, msg, wParam, lParam):
         delta = c_short(wParam >> 16).value
